package org.herac.tuxguitar.io.pdf;

import java.io.OutputStream;

import org.herac.tuxguitar.gui.TuxGuitar;
import org.herac.tuxguitar.gui.editors.tab.TGFactoryImpl;
import org.herac.tuxguitar.gui.editors.tab.Tablature;
import org.herac.tuxguitar.gui.editors.tab.layout.PrinterViewLayout;
import org.herac.tuxguitar.gui.helper.SyncThread;
import org.herac.tuxguitar.gui.printer.PrintDocument;
import org.herac.tuxguitar.gui.printer.PrintDocumentData;
import org.herac.tuxguitar.gui.util.MessageDialog;
import org.herac.tuxguitar.io.base.TGFileFormat;
import org.herac.tuxguitar.io.base.TGSongExporter;
import org.herac.tuxguitar.song.managers.TGSongManager;
import org.herac.tuxguitar.song.models.TGSong;

public class PDFSongExporter implements TGSongExporter{

	private static final int PAGE_WIDTH = 550;	
	private static final int PAGE_HEIGHT = 800;
	
	private PrintDocumentData data;
	
	public String getExportName() {
		return "PDF";
	}

	public TGFileFormat getFileFormat() {
		return new TGFileFormat("PDF","*.pdf");
	}	
	
	public boolean configure() {
		this.data = PrintDocumentData.openDialog(TuxGuitar.instance().getShell());
		return (this.data != null);
	}

	public void exportSong(OutputStream stream, TGSong song) {		
    	try{
    		if(this.data != null){
    			export(stream,song,this.data);
    		}
    	}catch(OutOfMemoryError error){
    		return;
    	}
	}

	public void export(final OutputStream stream,final TGSong song,final PrintDocumentData data){
		new Thread(new Runnable() {
			public void run() {
				try{
					TGSongManager manager = new TGSongManager();
					manager.setFactory(new TGFactoryImpl());
					manager.setSong(song.clone(manager.getFactory()));	
				
					export(stream,manager,data);
				}catch(Throwable throwable){
					MessageDialog.errorMessage(throwable);
				}				
			}
		}).start();
	}    
    
	public void export(final OutputStream stream,final TGSongManager manager, final PrintDocumentData data){	
		new SyncThread(new Runnable() {
			public void run() {
				try{
					Tablature tablature = new Tablature(TuxGuitar.instance().getShell());
			        tablature.setSongManager(manager);
		        
			        PrinterViewLayout layout = new PrinterViewLayout(tablature,data, 1f);
				
			        export(stream, layout);
				}catch(Throwable throwable){
					MessageDialog.errorMessage(throwable);
				}
			}
		}).start();  		        
	}

	public void export(final OutputStream stream, final PrinterViewLayout layout){	
		new Thread(new Runnable() {
			public void run() {
				try{	
			        layout.getTablature().updateTablature();	
			        layout.getTablature().getCaret().update(TuxGuitar.instance().getTablatureEditor().getTablature().getCaret().getTrack().getNumber());
					new SyncThread(new Runnable() {
						public void run() {
					        final PrintDocument doc = layout.makeDocument(PAGE_WIDTH,PAGE_HEIGHT);        
					        layout.getTablature().dispose();
							new Thread(new Runnable() {
								public void run() {
									try{
										PDFWriter.write(stream,doc);
									}catch(Throwable throwable){
										MessageDialog.errorMessage(throwable);
									}					        
								}
							}).start(); 
						}
					}).start(); 
				}catch(Throwable throwable){
					MessageDialog.errorMessage(throwable);
				}
			}
		}).start();  		
	}
}
